<?php
/**
 * Customizer
 *
 * @package auto-pt
 */

use ProteusThemes\CustomizerUtils\Setting;
use ProteusThemes\CustomizerUtils\Control;
use ProteusThemes\CustomizerUtils\CacheManager;

/**
 * Contains methods for customizing the theme customization screen.
 *
 * @link http://codex.wordpress.org/Theme_Customization_API
 */
class PT_Auto_Customizer_Base {
	/**
	 * The singleton manager instance
	 *
	 * @see wp-includes/class-wp-customize-manager.php
	 * @var WP_Customize_Manager
	 */
	protected $wp_customize;

	/**
	 * Instance of the DynamicCSS cache manager
	 * @var ProteusThemes\CustomizerUtils\CacheManager
	 */
	private $dynamic_css_cache_manager;

	/**
	 * Holds the array for the DynamiCSS.
	 */
	private $dynamic_css = array();

	public function __construct( WP_Customize_Manager $wp_manager ) {
		// Set the private propery to instance of wp_manager.
		$this->wp_customize = $wp_manager;

		// Set the private propery to instance of DynamicCSS CacheManager.
		$this->dynamic_css_cache_manager = new CacheManager( $this->wp_customize );

		// Init the dynamic_css property.
		$this->dynamic_css = $this->dynamic_css_init();

		// Register the settings/panels/sections/controls, main method.
		$this->register();
		$this->register_partials();

		/**
		 * Action and filters
		 */

		// Render the CSS and cache it to the theme_mod when the setting is saved.
		add_action( 'wp_head', array( 'ProteusThemes\CustomizerUtils\Helpers', 'add_dynamic_css_style_tag' ), 50, 0 );
		add_action( 'customize_save_after', function() {
			$this->dynamic_css_cache_manager->cache_rendered_css();
		} );

		// Save logo width/height dimensions.
		add_action( 'customize_save_logo_img' , array( __CLASS__, 'save_logo_dimensions' ), 10, 1 );
	}

	private function dynamic_css_init() {
		$darken5         = new Setting\DynamicCSS\ModDarken( 5 );
		$darken7         = new Setting\DynamicCSS\ModDarken( 7 );
		$darken12        = new Setting\DynamicCSS\ModDarken( 12 );
		$darken13        = new Setting\DynamicCSS\ModDarken( 13 );
		$lighten5        = new Setting\DynamicCSS\ModLighten( 5 );
		$lighten8        = new Setting\DynamicCSS\ModLighten( 8 );
		$light_gradient  = new Setting\DynamicCSS\ModLinearGradient( $darken7, $orientation = 'to bottom' );
		$button_gradient = new Setting\DynamicCSS\ModLinearGradient( $lighten5, $orientation = 'to top' );

		return array(
			'header_bg' => array(
				'default'    => '#ffffff',
				'css_props' => array(
					array(
						'name'      => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.header__container',
							),
						),
					),
					array(
						'name'      => 'background-color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.header__widgets',
							),
						),
						'modifier'  => $darken7,
					),
					array(
						'name'      => 'background-color',
						'selectors' => array(
							'@media (max-width: 991px)' => array(
								'.header .icon-box',
							),
						),
						'modifier'  => $darken7,
					),
					array(
						'name'      => 'background',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.header__navigation',
								'.header__navigation-widgets',
							),
						),
						'modifier'  => $light_gradient,
					),
				),
			),

			'main_navigation_mobile_background' => array(
				'default' => '#ffffff',
				'css_props' => array(
					array(
						'name' => 'background-color',
						'selectors' => array(
							'@media (max-width: 991px)' => array(
								'.main-navigation',
							),
						),
					),
					array(
						'name' => 'border-color',
						'selectors' => array(
							'@media (max-width: 991px)' => array(
								'.main-navigation',
								'.main-navigation a',
							),
						),
						'modifier'  => $darken13,
					),
				),
			),

			'main_navigation_mobile_color' => array(
				'default' => '#34373b',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'@media (max-width: 991px)' => array(
								'.main-navigation a',
							),
						),
					),
				),
			),

			'main_navigation_mobile_color_hover' => array(
				'default' => '#bc001f',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'@media (max-width: 991px)' => array(
								'.main-navigation .menu-item:focus > a',
								'.main-navigation .menu-item:hover > a',
							),
						),
					),
				),
			),

			'main_navigation_mobile_sub_color_background' => array(
				'default' => '#eeeeee',
				'css_props' => array(
					array(
						'name' => 'background-color',
						'selectors' => array(
							'@media (max-width: 991px)' => array(
								'.main-navigation .sub-menu',
							),
						),
					),
				),
			),

			'main_navigation_mobile_sub_color' => array(
				'default' => '#999999',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'@media (max-width: 991px)' => array(
								'.main-navigation .sub-menu a',
							),
						),
					),
				),
			),

			'main_navigation_mobile_sub_color_hover' => array(
				'default' => '#222222',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'@media (max-width: 991px)' => array(
								'.main-navigation .sub-menu .menu-item:hover > a',
								'.main-navigation .sub-menu .menu-item:focus > a',
							),
						),
					),
				),
			),

			'main_navigation_color' => array(
				'default' => '#34373b',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.main-navigation a',
								'.main-navigation > .menu-item-has-children::after',
							),
						),
					),
				),
			),

			'main_navigation_color_hover' => array(
				'default' => '#bc001f',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.main-navigation .menu-item:focus > a',
								'.main-navigation .menu-item:hover > a',
								'.main-navigation > .current-menu-item > a',
								'.main-navigation .menu-item:focus::after',
								'.main-navigation .menu-item:hover::after',
								'.main-navigation > .current-menu-item.menu-item-has-children::after',
							),
						),
					),
					array(
						'name' => 'background-color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.main-navigation > .current-menu-item > a::after',
							),
						),
					),
				),
			),

			'main_navigation_sub_bg' => array(
				'default' => '#222222',
				'css_props' => array(
					array(
						'name' => 'background-color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.main-navigation .sub-menu a',
							),
						),
					),
					array(
						'name' => 'border-color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.main-navigation .sub-menu a',
								'.main-navigation .sub-menu .sub-menu a',
							),
						),
						'modifier'  => $lighten8,
					),
					array(
						'name' => 'color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.main-navigation .sub-menu .menu-item-has-children::after',
							),
						),
						'modifier'  => $lighten8,
					),
				),
			),

			'main_navigation_sub_color' => array(
				'default' => '#777777',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.main-navigation .sub-menu a',
							),
						),
					),
				),
			),

			'main_navigation_sub_color_hover' => array(
				'default' => '#bbbbbb',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.main-navigation .sub-menu .menu-item:focus > a',
								'.main-navigation .sub-menu .menu-item:hover > a',
							),
						),
					),
				),
			),

			'page_header_bg_color' => array(
				'default' => '#313336',
				'css_props' => array(
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.site-header',
							),
						),
					),
				),
			),

			'page_header_color' => array(
				'default' => '#ffffff',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'.page-header__title',
							),
						),
					),
				),
			),

			'page_header_subtitle_color' => array(
				'default' => '#999999',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'.page-header__subtitle',
							),
						),
					),
				),
			),

			'breadcrumbs_bg' => array(
				'default'    => '#ffffff',
				'css_props' => array(
					array(
						'name'      => 'background',
						'selectors' => array(
							'noop' => array(
								'.breadcrumbs',
							),
						),
						'modifier'  => $light_gradient,
					),
					array(
						'name'      => 'border-color',
						'selectors' => array(
							'noop' => array(
								'.breadcrumbs',
							),
						),
						'modifier'  => $darken12,
					),
				),
			),

			'breadcrumbs_color' => array(
				'default'    => '#999999',
				'css_props' => array(
					array(
						'name'      => 'color',
						'selectors' => array(
							'noop' => array(
								'.breadcrumbs a',
							),
						),
					),
				),
			),

			'breadcrumbs_color_hover' => array(
				'default'    => '#34373b',
				'css_props' => array(
					array(
						'name'      => 'color',
						'selectors' => array(
							'noop' => array(
								'.breadcrumbs a:focus',
								'.breadcrumbs a:hover',
							),
						),
					),
				),
			),

			'breadcrumbs_color_active' => array(
				'default'    => '#34373b',
				'css_props' => array(
					array(
						'name'      => 'color',
						'selectors' => array(
							'noop' => array(
								'.breadcrumbs .current-item',
							),
						),
					),
				),
			),

			'text_color_content_area' => array(
				'default' => '#999999',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'.content-area',
							),
						),
					),
				),
			),

			'headings_color' => array(
				'default' => '#34373b',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'h1',
								'h2',
								'h3',
								'h4',
								'h5',
								'h6',
								'hentry__title',
								'.hentry__title a',
								'.page-box__title a',
								'.latest-news--block .latest-news__title a',
								'.accordion__panel .panel-title a',
								'.icon-menu__link',
								'.step__title',
								'body.woocommerce-page ul.products li.product h3',
								'.woocommerce ul.products li.product h3',
							),
						),
					),
				),
			),

			'primary_color' => array(
				'default' => '#bc001f',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'body.woocommerce-page div.product .woocommerce-tabs ul.tabs li.active a',
								'body.woocommerce-page ul.products li.product a',
								'body.woocommerce-page ul.products li.product a:hover img',
								'.woocommerce ul.products li.product a',
								'.woocommerce ul.products li.product a:hover img',
								'body.woocommerce-page ul.products li.product .price',
								'.woocommerce ul.products li.product .price',
								'body.woocommerce-page .star-rating',
								'.woocommerce .star-rating',
								'body.woocommerce-page div.product p.price',
								'body.woocommerce-page p.stars a',
								'body.woocommerce-page ul.product_list_widget .amount',
								'.woocommerce.widget_shopping_cart .total .amount',
								'.social-icons__link:hover',
								'.header a.icon-box:hover .fa',
								'.header a.icon-box:hover .icon-box__title',
								'.header a.icon-box:hover .icon-box__subtitle',
								'.portfolio-grid__nav-item.is-active > .portfolio-grid__nav-link',
								'.portfolio-grid__card:focus .portfolio-grid__card-title',
								'.portfolio-grid__card:hover .portfolio-grid__card-title',
								'.latest-news--more-news',
								'.latest-news--more-news:focus',
								'.latest-news--more-news:hover',
								'.testimonial__author-description',
								'.person-profile__social-icon:hover',
								'.footer-navigation a:focus',
								'.footer-navigation a:hover',
								'.portfolio-grid__nav-item.is-active > .portfolio-grid__nav-link:focus',
								'.portfolio-grid__nav-item.is-active > .portfolio-grid__nav-link:hover',
								'body.woocommerce-page .widget_product_categories .product-categories li.current-cat > a',
								'.portfolio-grid__card-price',
								'.hentry__content a.icon-box:focus .fa, .hentry__content a.icon-box:hover .fa',
								'.hentry__content .widget_nav_menu .menu li.current-menu-item > a',
								'.sidebar .widget_nav_menu .menu li.current-menu-item > a',
							),
						),
					),
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'body.woocommerce-page nav.woocommerce-pagination ul li span.current',
								'body.woocommerce-page .widget_product_search .search-field + input',
								'body.woocommerce-page div.product .woocommerce-tabs ul.tabs li.active a::after',
								'body.woocommerce-page div.product .woocommerce-tabs ul.tabs li:hover a::after',
								'body.woocommerce-page .widget_price_filter .ui-slider .ui-slider-handle',
								'body.woocommerce-page .widget_price_filter .ui-slider .ui-slider-range',
								'.step:hover .step__number',
								'.portfolio-grid__nav-link:focus::after, .portfolio-grid__nav-link:hover::after',
								'.latest-news--block .latest-news__date',
								'.person-profile__tag',
								'.pricing-list__badge',
								'.page-box__tag',
								'.hentry__date',
								'.meta__item--date',
								'.pagination .current',
								'.widget_calendar caption',
							),
						),
					),
					array(
						'name' => 'background',
						'selectors' => array(
							'noop' => array(
								'body.woocommerce-page .widget_shopping_cart_content .buttons .checkout',
								'body.woocommerce-page button.button.alt',
								'body.woocommerce-page .woocommerce-error a.button',
								'body.woocommerce-page .woocommerce-info a.button',
								'body.woocommerce-page .woocommerce-message a.button',
								'.woocommerce-cart .wc-proceed-to-checkout a.checkout-button',
								'body.woocommerce-page #payment #place_order',
								'body.woocommerce-page #review_form #respond input#submit',
								'body.woocommerce-page div.product form.cart .button.single_add_to_cart_button',
								'.btn-primary',
								'.header .icon-box--featured',
							),
						),
						'modifier'  => $button_gradient,
					),
					array(
						'name' => 'background',
						'selectors' => array(
							'noop' => array(
								'body.woocommerce-page div.product form.cart .button.single_add_to_cart_button:hover',
								'body.woocommerce-page div.product form.cart .button.single_add_to_cart_button:focus',
								'.btn-primary:hover',
								'.btn-primary:focus',
								'body.woocommerce-page .woocommerce-error a.button:hover',
								'body.woocommerce-page .woocommerce-info a.button:hover',
								'body.woocommerce-page .woocommerce-message a.button:hover',
								'.woocommerce button.button.alt:disabled',
								'.woocommerce button.button.alt:disabled:hover',
								'.woocommerce button.button.alt:disabled[disabled]',
								'.woocommerce button.button.alt:disabled[disabled]:hover',
								'.woocommerce-cart .wc-proceed-to-checkout a.checkout-button:hover',
								'body.woocommerce-page #review_form #respond input#submit:hover',
								'body.woocommerce-page .widget_product_search .search-field + input:hover',
								'body.woocommerce-page .widget_product_search .search-field + input:focus',
							),
						),
					),
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.btn-primary:active:focus',
							),
						),
						'modifier'  => $darken7,
					),
					array(
						'name' => 'border-color',
						'selectors' => array(
							'noop' => array(
								'body.woocommerce-page .widget_shopping_cart_content .buttons .checkout',
								'body.woocommerce-page nav.woocommerce-pagination ul li span.current',
							),
						),
					),
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(),
						),
					),
					array(
						'name' => 'border-color',
						'selectors' => array(
							'noop' => array(),
						),
					),
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(),
						),
					),
				),
			),

			'link_color' => array(
				'default' => '#bc001f',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'a',
							),
						),
					),
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'a:focus',
								'a:hover',
							),
						),
						'modifier'  => $darken7,
					),
				),
			),

			'dark_button_color' => array(
				'default' => '#34373b',
				'css_props' => array(
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.btn-dark',
							),
						),
					),
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.btn-dark:focus',
								'.btn-dark:hover',
							),
						),
						'modifier' => $darken5,
					),
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.btn-dark:active:focus',
							),
						),
						'modifier' => $darken7,
					),
				),
			),

			'light_button_color' => array(
				'default' => '#f2f2f2',
				'css_props' => array(
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.btn-light',
							),
						),
					),
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.btn-light:focus',
								'.btn-light:hover',
							),
						),
						'modifier' => $darken5,
					),
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.btn-light:active:focus',
							),
						),
						'modifier' => $darken7,
					),
				),
			),

			'footer_bg_color' => array(
				'default' => '#eeeeee',
				'css_props' => array(
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.footer-top',
							),
						),
					),
				),
			),

			'footer_title_color' => array(
				'default' => '#34373b',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'.footer-top__headings',
							),
						),
					),
				),
			),

			'footer_text_color' => array(
				'default' => '#999999',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'.footer-top',
							),
						),
					),
				),
			),

			'footer_link_color' => array(
				'default' => '#999999',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'.footer-top .widget_nav_menu .menu a',
							),
						),
					),
				),
			),

			'footer_middle_bg_color' => array(
				'default' => '#ffffff',
				'css_props' => array(
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.footer-middle__container',
							),
						),
					),
				),
			),

			'footer_middle_text_color' => array(
				'default' => '#999999',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'.footer-middle',
								'.footer-navigation a',
							),
						),
					),
				),
			),

			'footer_bottom_bg_color' => array(
				'default' => '#eeeeee',
				'css_props' => array(
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.footer-bottom__container',
							),
						),
					),
				),
			),

			'footer_bottom_text_color' => array(
				'default' => '#999999',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'.footer-bottom',
							),
						),
					),
				),
			),

			'footer_bottom_link_color' => array(
				'default' => '#34373b',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'.footer-bottom a',
							),
						),
					),
				),
			),

		);
	}

	/**
	 * This hooks into 'customize_register' (available as of WP 3.4) and allows
	 * you to add new sections and controls to the Theme Customize screen.
	 *
	 * Note: To enable instant preview, we have to actually write a bit of custom
	 * javascript. See live_preview() for more.
	 *
	 * @see add_action('customize_register',$func)
	 */
	public function register() {
		/**
		 * Settings
		 */

		// Branding.
		$this->wp_customize->add_setting( 'logo_img' );
		$this->wp_customize->add_setting( 'logo2x_img' );
		$this->wp_customize->add_setting( 'logo_top_margin', array( 'default' => 27 ) );

		// Page header area.
		$this->wp_customize->add_setting( 'page_header_bg_img', array( 'default' => get_template_directory_uri() . '/assets/images/dark-pattern.jpg' ) );
		$this->wp_customize->add_setting( 'page_header_bg_img_repeat', array( 'default' => 'repeat' ) );
		$this->wp_customize->add_setting( 'page_header_bg_img_position_x', array( 'default' => 'left' ) );
		$this->wp_customize->add_setting( 'page_header_bg_img_attachment', array( 'default' => 'scroll' ) );

		// Typography.
		$this->wp_customize->add_setting( 'charset_setting', array( 'default' => 'latin' ) );

		// Theme layout & color.
		$this->wp_customize->add_setting( 'layout_mode', array( 'default' => 'wide' ) );

		// Shop.
		if ( AutoPTHelpers::is_woocommerce_active() ) {
			$this->wp_customize->add_setting( 'products_per_page', array( 'default' => 9 ) );
			$this->wp_customize->add_setting( 'single_product_sidebar', array( 'default' => 'left' ) );
		}

		// Portfolio.
		if ( AutoPTHelpers::is_portfolio_plugin_active() ) {
			$this->wp_customize->add_setting( 'portfolio_parent_page', array( 'default' => 0 ) );
			$this->wp_customize->add_setting( 'portfolio_gallery_columns', array( 'default' => 2 ) );
			$this->wp_customize->add_setting( 'portfolio_slug', array(
				'default'           => 'used-cars',
				'sanitize_callback' => array( $this, 'sanitize_portfolio_slug' ),
			) );
		}

		// Footer.
		$this->wp_customize->add_setting( 'footer_widgets_layout', array( 'default' => '[6,9]' ) );

		$this->wp_customize->add_setting( 'footer_bottom_left_txt', array( 'default' => '<a href="https://www.proteusthemes.com/wordpress-themes/auto/">Auto Theme</a> - Made by ProteusThemes.' ) );
		$this->wp_customize->add_setting( 'footer_bottom_right_txt', array( 'default' => 'Copyright &copy; 2009&#8211;2016 Auto. All rights reserved.' ) );

		$this->wp_customize->add_setting( 'footer_middle_left_txt', array( 'default' => 'We accept credit cards &nbsp; &nbsp; &nbsp; <span class="footer__icons"><i class="fa  fa-3x  fa-cc-visa"></i> &nbsp; <i class="fa  fa-3x  fa-cc-mastercard"></i> &nbsp; <i class="fa  fa-3x  fa-cc-amex"></i> &nbsp; <i class="fa  fa-3x fa-cc-paypal"></i></span>' ) );
		$this->wp_customize->add_setting( 'footer_middle_right_txt', array( 'default' => '' ) );

		$this->wp_customize->add_setting( 'footer_back_to_top', array( 'default' => 'yes' ) );

		// Custom code (css/js).
		$this->wp_customize->add_setting( 'custom_js_head' );
		$this->wp_customize->add_setting( 'custom_js_footer' );
		$this->wp_customize->add_setting( 'custom_css', array( 'default' => '' ) );

		// Migrate any existing theme CSS to the core option added in WordPress 4.7.
		if ( function_exists( 'wp_update_custom_css_post' ) ) {
			$css = get_theme_mod( 'custom_css', '' );

			if ( ! empty( $css ) ) {
				$core_css = wp_get_custom_css(); // Preserve any CSS already added to the core option.
				$return   = wp_update_custom_css_post( '/* Migrated CSS from old Theme Custom CSS setting: */' . PHP_EOL . $css . PHP_EOL . PHP_EOL . '/* New custom CSS: */' . PHP_EOL . $core_css );
				if ( ! is_wp_error( $return ) ) {
					// Remove the old theme_mod, so that the CSS is stored in only one place moving forward.
					remove_theme_mod( 'custom_css' );
				}
			}

			// Add new "CSS setting" that will only notify the users that the new "Additional CSS" field is available.
			// It can't be the same name ('custom_css'), because the core control is also named 'custom_css' and it would not display the WP core "Additional CSS" control.
			$this->wp_customize->add_setting( 'pt_custom_css', array( 'default' => '' ) );
		}

		// ACF.
		$this->wp_customize->add_setting( 'show_acf', array( 'default' => 'no' ) );
		$this->wp_customize->add_setting( 'use_minified_css', array( 'default' => 'no' ) );

		// All the DynamicCSS settings.
		foreach ( $this->dynamic_css as $setting_id => $args ) {
			$this->wp_customize->add_setting(
				new Setting\DynamicCSS( $this->wp_customize, $setting_id, $args )
			);
		}

		/**
		 * Panel and Sections
		 */

		// One ProteusThemes panel to rule them all.
		$this->wp_customize->add_panel( 'panel_autopt', array(
			'title'       => esc_html__( '[PT] Theme Options', 'auto-pt' ),
			'description' => esc_html__( 'All Auto theme specific settings.', 'auto-pt' ),
			'priority'    => 10,
		) );

		// Individual sections.
		$this->wp_customize->add_section( 'autopt_section_logos', array(
			'title'       => esc_html__( 'Logo', 'auto-pt' ),
			'description' => esc_html__( 'Logo for the Auto theme.', 'auto-pt' ),
			'priority'    => 10,
			'panel'       => 'panel_autopt',
		) );

		// Header.
		$this->wp_customize->add_section( 'autopt_section_header', array(
			'title'       => esc_html__( 'Header', 'auto-pt' ),
			'description' => esc_html__( 'All layout and appearance settings for the header.', 'auto-pt' ),
			'priority'    => 20,
			'panel'       => 'panel_autopt',
		) );

		$this->wp_customize->add_section( 'autopt_section_navigation', array(
			'title'       => esc_html__( 'Navigation', 'auto-pt' ),
			'description' => esc_html__( 'Navigation for the Auto theme.', 'auto-pt' ),
			'priority'    => 30,
			'panel'       => 'panel_autopt',
		) );

		$this->wp_customize->add_section( 'autopt_section_page_header', array(
			'title'       => esc_html__( 'Page Header Area', 'auto-pt' ),
			'description' => esc_html__( 'All layout and appearance settings for the page header area (regular pages).', 'auto-pt' ),
			'priority'    => 33,
			'panel'       => 'panel_autopt',
		) );

		$this->wp_customize->add_section( 'autopt_section_breadcrumbs', array(
			'title'       => esc_html__( 'Breadcrumbs', 'auto-pt' ),
			'description' => esc_html__( 'All layout and appearance settings for breadcrumbs.', 'auto-pt' ),
			'priority'    => 35,
			'panel'       => 'panel_autopt',
		) );

		$this->wp_customize->add_section( 'autopt_section_theme_colors', array(
			'title'       => esc_html__( 'Theme Layout &amp; Colors', 'auto-pt' ),
			'priority'    => 40,
			'panel'       => 'panel_autopt',
		) );

		if ( AutoPTHelpers::is_woocommerce_active() ) {
			$this->wp_customize->add_section( 'autopt_section_shop', array(
				'title'       => esc_html__( 'Shop', 'auto-pt' ),
				'priority'    => 80,
				'panel'       => 'panel_autopt',
			) );
		}

		if ( AutoPTHelpers::is_portfolio_plugin_active() ) {
			$this->wp_customize->add_section( 'autopt_section_portfolio', array(
				'title'       => esc_html__( 'Portfolio', 'auto-pt' ),
				'priority'    => 85,
				'panel'       => 'panel_autopt',
			) );
		}

		$this->wp_customize->add_section( 'section_footer', array(
			'title'       => esc_html__( 'Footer', 'auto-pt' ),
			'description' => esc_html__( 'All layout and appearance settings for the footer.', 'auto-pt' ),
			'priority'    => 90,
			'panel'       => 'panel_autopt',
		) );

		$this->wp_customize->add_section( 'section_custom_code', array(
			'title'       => esc_html__( 'Custom Code' , 'auto-pt' ),
			'priority'    => 100,
			'panel'       => 'panel_autopt',
		) );

		$this->wp_customize->add_section( 'section_other', array(
			'title'       => esc_html__( 'Other' , 'auto-pt' ),
			'priority'    => 150,
			'panel'       => 'panel_autopt',
		) );

		/**
		 * Controls
		 */

		// Section: autopt_section_logos.
		$this->wp_customize->add_control( new WP_Customize_Image_Control(
			$this->wp_customize,
			'logo_img',
			array(
				'label'       => esc_html__( 'Logo Image', 'auto-pt' ),
				'description' => esc_html__( 'Max height for the logo image is 120px.', 'auto-pt' ),
				'section'     => 'autopt_section_logos',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Image_Control(
			$this->wp_customize,
			'logo2x_img',
			array(
				'label'       => esc_html__( 'Retina Logo Image', 'auto-pt' ),
				'description' => esc_html__( '2x logo size, for screens with high DPI.', 'auto-pt' ),
				'section'     => 'autopt_section_logos',
			)
		) );
		$this->wp_customize->add_control(
			'logo_top_margin',
			array(
				'type'        => 'number',
				'label'       => esc_html__( 'Logo top margin', 'auto-pt' ),
				'description' => esc_html__( 'In pixels.', 'auto-pt' ),
				'section'     => 'autopt_section_logos',
				'input_attrs' => array(
					'min'  => 0,
					'max'  => 120,
					'step' => 5,
				),
			)
		);

		// Section: header.
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'header_bg',
			array(
				'priority' => 30,
				'label'    => esc_html__( 'Header background color', 'auto-pt' ),
				'section'  => 'autopt_section_header',
			)
		) );

		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_color',
			array(
				'priority' => 130,
				'label'    => esc_html__( 'Main navigation link color', 'auto-pt' ),
				'section'  => 'autopt_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_color_hover',
			array(
				'priority' => 132,
				'label'    => esc_html__( 'Main navigation link hover color', 'auto-pt' ),
				'section'  => 'autopt_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_sub_bg',
			array(
				'priority' => 160,
				'label'    => esc_html__( 'Main navigation submenu background', 'auto-pt' ),
				'section'  => 'autopt_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_sub_color',
			array(
				'priority' => 170,
				'label'    => esc_html__( 'Main navigation submenu link color', 'auto-pt' ),
				'section'  => 'autopt_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_sub_color_hover',
			array(
				'priority' => 175,
				'label'    => esc_html__( 'Main navigation submenu link hover color', 'auto-pt' ),
				'section'  => 'autopt_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_mobile_color',
			array(
				'priority' => 190,
				'label'    => esc_html__( 'Main navigation link color (mobile)', 'auto-pt' ),
				'section'  => 'autopt_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_mobile_color_hover',
			array(
				'priority' => 192,
				'label'    => esc_html__( 'Main navigation link hover color (mobile)', 'auto-pt' ),
				'section'  => 'autopt_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_mobile_sub_color_background',
			array(
				'priority' => 193,
				'label'    => esc_html__( 'Main navigation submenu background color (mobile)', 'auto-pt' ),
				'section'  => 'autopt_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_mobile_sub_color',
			array(
				'priority' => 194,
				'label'    => esc_html__( 'Main navigation submenu link color (mobile)', 'auto-pt' ),
				'section'  => 'autopt_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_mobile_sub_color_hover',
			array(
				'priority' => 195,
				'label'    => esc_html__( 'Main navigation submenu link hover color (mobile)', 'auto-pt' ),
				'section'  => 'autopt_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_mobile_background',
			array(
				'priority' => 188,
				'label'    => esc_html__( 'Main navigation background color (mobile)', 'auto-pt' ),
				'section'  => 'autopt_section_navigation',
			)
		) );

		// Section: autopt_section_page_header.
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'page_header_bg_color',
			array(
				'priority' => 10,
				'label'    => esc_html__( 'Page Header background color', 'auto-pt' ),
				'section'  => 'autopt_section_page_header',
			)
		) );

		$this->wp_customize->add_control( new WP_Customize_Image_Control(
			$this->wp_customize,
			'page_header_bg_img',
			array(
				'priority' => 20,
				'label'    => esc_html__( 'Page Header Background Image', 'auto-pt' ),
				'section'  => 'autopt_section_page_header',
			)
		) );
		$this->wp_customize->add_control( 'page_header_bg_img_repeat', array(
			'priority'   => 21,
			'label'      => esc_html__( 'Page Header Background Repeat', 'auto-pt' ),
			'section'    => 'autopt_section_page_header',
			'type'       => 'radio',
			'choices'    => array(
				'no-repeat'  => esc_html__( 'No Repeat', 'auto-pt' ),
				'repeat'     => esc_html__( 'Tile', 'auto-pt' ),
				'repeat-x'   => esc_html__( 'Tile Horizontally', 'auto-pt' ),
				'repeat-y'   => esc_html__( 'Tile Vertically', 'auto-pt' ),
			),
		) );
		$this->wp_customize->add_control( 'page_header_bg_img_position_x', array(
			'priority'   => 22,
			'label'      => esc_html__( 'Page Header Background Position', 'auto-pt' ),
			'section'    => 'autopt_section_page_header',
			'type'       => 'radio',
			'choices'    => array(
				'left'   => esc_html__( 'Left', 'auto-pt' ),
				'center' => esc_html__( 'Center', 'auto-pt' ),
				'right'  => esc_html__( 'Right', 'auto-pt' ),
			),
		) );
		$this->wp_customize->add_control( 'page_header_bg_img_attachment', array(
			'priority'   => 23,
			'label'      => esc_html__( 'Page Header Background Attachment', 'auto-pt' ),
			'section'    => 'autopt_section_page_header',
			'type'       => 'radio',
			'choices'    => array(
				'scroll' => esc_html__( 'Scroll', 'auto-pt' ),
				'fixed'  => esc_html__( 'Fixed', 'auto-pt' ),
			),
		) );

		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'page_header_color',
			array(
				'priority' => 30,
				'label'    => esc_html__( 'Page Header title color', 'auto-pt' ),
				'section'  => 'autopt_section_page_header',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'page_header_subtitle_color',
			array(
				'priority' => 31,
				'label'    => esc_html__( 'Page Header subtitle color', 'auto-pt' ),
				'section'  => 'autopt_section_page_header',
			)
		) );

		// Section: autopt_section_breadcrumbs.
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'breadcrumbs_bg',
			array(
				'priority' => 10,
				'label'    => esc_html__( 'Breadcrumbs background color', 'auto-pt' ),
				'section'  => 'autopt_section_breadcrumbs',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'breadcrumbs_color',
			array(
				'priority' => 45,
				'label'    => esc_html__( 'Breadcrumbs text color', 'auto-pt' ),
				'section'  => 'autopt_section_breadcrumbs',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'breadcrumbs_color_hover',
			array(
				'priority' => 50,
				'label'    => esc_html__( 'Breadcrumbs hover text color', 'auto-pt' ),
				'section'  => 'autopt_section_breadcrumbs',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'breadcrumbs_color_active',
			array(
				'priority'    => 50,
				'label'       => esc_html__( 'Breadcrumbs active text color', 'auto-pt' ),
				'description' => esc_html__( 'Only works when "Link Current Item" is enabled in the Breadcrumbs NavXT settings.', 'auto-pt' ),
				'section'     => 'autopt_section_breadcrumbs',
			)
		) );

		// Section: autopt_section_theme_colors.
		$this->wp_customize->add_control( 'layout_mode', array(
			'type'     => 'select',
			'priority' => 10,
			'label'    => esc_html__( 'Layout', 'auto-pt' ),
			'section'  => 'autopt_section_theme_colors',
			'choices'  => array(
				'wide'  => esc_html__( 'Wide', 'auto-pt' ),
				'boxed' => esc_html__( 'Boxed', 'auto-pt' ),
			),
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'text_color_content_area',
			array(
				'priority' => 30,
				'label'    => esc_html__( 'Text color', 'auto-pt' ),
				'section'  => 'autopt_section_theme_colors',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'link_color',
			array(
				'priority' => 35,
				'label'    => esc_html__( 'Link color', 'auto-pt' ),
				'section'  => 'autopt_section_theme_colors',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'headings_color',
			array(
				'priority' => 33,
				'label'    => esc_html__( 'Headings color', 'auto-pt' ),
				'section'  => 'autopt_section_theme_colors',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'primary_color',
			array(
				'priority' => 34,
				'label'    => esc_html__( 'Primary color', 'auto-pt' ),
				'section'  => 'autopt_section_theme_colors',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'dark_button_color',
			array(
				'priority' => 36,
				'label'    => esc_html__( 'Dark button background color', 'auto-pt' ),
				'section'  => 'autopt_section_theme_colors',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'light_button_color',
			array(
				'priority' => 37,
				'label'    => esc_html__( 'Light button background color', 'auto-pt' ),
				'section'  => 'autopt_section_theme_colors',
			)
		) );

		// Section: autopt_section_shop.
		if ( AutoPTHelpers::is_woocommerce_active() ) {
			$this->wp_customize->add_control( 'products_per_page', array(
					'label'   => esc_html__( 'Number of products per page', 'auto-pt' ),
					'section' => 'autopt_section_shop',
				)
			);
			$this->wp_customize->add_control( 'single_product_sidebar', array(
					'label'   => esc_html__( 'Sidebar on single product page', 'auto-pt' ),
					'section' => 'autopt_section_shop',
					'type'    => 'select',
					'choices' => array(
						'none'  => esc_html__( 'No sidebar', 'auto-pt' ),
						'left'  => esc_html__( 'Left', 'auto-pt' ),
						'right' => esc_html__( 'Right', 'auto-pt' ),
					),
				)
			);
		}

		// Section: autopt_section_portfolio.
		if ( AutoPTHelpers::is_portfolio_plugin_active() ) {
			$this->wp_customize->add_control( 'portfolio_parent_page', array(
					'label'       => esc_html__( 'Portfolio parent page', 'auto-pt' ),
					'description' => esc_html__( 'Page with all the portfolio items.', 'auto-pt' ),
					'section'     => 'autopt_section_portfolio',
					'type'        => 'dropdown-pages',
				)
			);
			$this->wp_customize->add_control( 'portfolio_slug', array(
					'label'       => esc_html__( 'Portfolio slug', 'auto-pt' ),
					'description' => esc_html__( 'This is used in the URL part. After changing this, you must save the permalink settings again in Settings &rarr; Permalinks.', 'auto-pt' ),
					'section'     => 'autopt_section_portfolio',
				)
			);
			$this->wp_customize->add_control( 'portfolio_gallery_columns', array(
					'label'   => esc_html__( 'Number of columns in gallery', 'auto-pt' ),
					'section' => 'autopt_section_portfolio',
					'type'    => 'select',
					'choices' => array(
						'1' => '1',
						'2' => '2',
						'3' => '3',
					),
				)
			);
		}

		// Section: section_footer.
		$this->wp_customize->add_control( new Control\LayoutBuilder(
			$this->wp_customize,
			'footer_widgets_layout',
			array(
				'priority'    => 1,
				'label'       => esc_html__( 'Footer widgets layout', 'auto-pt' ),
				'description' => esc_html__( 'Select number of widget you want in the footer and then with the slider rearrange the layout', 'auto-pt' ),
				'section'     => 'section_footer',
				'input_attrs' => array(
					'min'     => 0,
					'max'     => 12,
					'step'    => 1,
					'maxCols' => 6,
				),
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'footer_bg_color',
			array(
				'priority' => 10,
				'label'    => esc_html__( 'Footer background color', 'auto-pt' ),
				'section'  => 'section_footer',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'footer_title_color',
			array(
				'priority' => 30,
				'label'    => esc_html__( 'Footer widget title color', 'auto-pt' ),
				'section'  => 'section_footer',
			)
		) );

		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'footer_text_color',
			array(
				'priority' => 31,
				'label'    => esc_html__( 'Footer text color', 'auto-pt' ),
				'section'  => 'section_footer',
			)
		) );

		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'footer_link_color',
			array(
				'priority' => 32,
				'label'    => esc_html__( 'Footer link color', 'auto-pt' ),
				'section'  => 'section_footer',
			)
		) );

		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'footer_middle_bg_color',
			array(
				'priority' => 33,
				'label'    => esc_html__( 'Footer middle background color', 'auto-pt' ),
				'section'  => 'section_footer',
			)
		) );

		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'footer_middle_text_color',
			array(
				'priority' => 34,
				'label'    => esc_html__( 'Footer middle text color', 'auto-pt' ),
				'section'  => 'section_footer',
			)
		) );

		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'footer_bottom_bg_color',
			array(
				'priority' => 35,
				'label'    => esc_html__( 'Footer bottom background color', 'auto-pt' ),
				'section'  => 'section_footer',
			)
		) );

		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'footer_bottom_text_color',
			array(
				'priority' => 36,
				'label'    => esc_html__( 'Footer bottom text color', 'auto-pt' ),
				'section'  => 'section_footer',
			)
		) );

		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'footer_bottom_link_color',
			array(
				'priority' => 37,
				'label'    => esc_html__( 'Footer bottom link color', 'auto-pt' ),
				'section'  => 'section_footer',
			)
		) );

		$this->wp_customize->add_control( 'footer_back_to_top', array(
			'type'        => 'select',
			'priority'    => 90,
			'label'       => esc_html__( 'Show "Back to top" button?', 'auto-pt' ),
			'section'     => 'section_footer',
			'choices'     => array(
				'yes' => esc_html__( 'Yes', 'auto-pt' ),
				'no'  => esc_html__( 'No', 'auto-pt' ),
			),
		) );

		$this->wp_customize->add_control( 'footer_middle_left_txt', array(
			'type'        => 'text',
			'priority'    => 100,
			'label'       => esc_html__( 'Footer middle text on the left', 'auto-pt' ),
			'description' => sprintf( esc_html__( 'You can use HTML: %s.', 'auto-pt' ), 'a, span, i, em, strong, img' ),
			'section'     => 'section_footer',
		) );

		$this->wp_customize->add_control( 'footer_middle_right_txt', array(
			'type'        => 'text',
			'priority'    => 105,
			'label'       => esc_html__( 'Footer middle text on the right', 'auto-pt' ),
			'description' => sprintf( esc_html__( 'You can use HTML: %s.', 'auto-pt' ), 'a, span, i, em, strong, img' ),
			'section'     => 'section_footer',
		) );

		$this->wp_customize->add_control( 'footer_bottom_left_txt', array(
			'type'        => 'text',
			'priority'    => 110,
			'label'       => esc_html__( 'Footer bottom text on the left', 'auto-pt' ),
			'description' => sprintf( esc_html__( 'You can use HTML: %s.', 'auto-pt' ), 'a, span, i, em, strong, img' ),
			'section'     => 'section_footer',
		) );

		$this->wp_customize->add_control( 'footer_bottom_right_txt', array(
			'type'        => 'text',
			'priority'    => 115,
			'label'       => esc_html__( 'Footer bottom text on the right', 'auto-pt' ),
			'description' => sprintf( esc_html__( 'You can use HTML: %s.', 'auto-pt' ), 'a, span, i, em, strong, img' ),
			'section'     => 'section_footer',
		) );

		// Section: section_custom_code.
		if ( function_exists( 'wp_update_custom_css_post' ) ) {
			// Show the notice of custom CSS setting migration.
			$this->wp_customize->add_control( 'pt_custom_css', array(
				'type'        => 'hidden',
				'label'       => esc_html__( 'Custom CSS', 'auto-pt' ),
				'description' => esc_html__( 'This field is obsolete. The existing code was migrated to the "Additional CSS" field, that can be found in the root of the customizer. This new "Additional CSS" field is a WordPress core field and was introduced in WP version 4.7.', 'auto-pt' ),
				'section'     => 'section_custom_code',
			) );
		}
		else {
			$this->wp_customize->add_control( 'custom_css', array(
				'type'        => 'textarea',
				'label'       => esc_html__( 'Custom CSS', 'auto-pt' ),
				'description' => sprintf( esc_html__( '%s How to find CSS classes %s in the theme.', 'auto-pt' ), '<a href="https://www.youtube.com/watch?v=V2aAEzlvyDc" target="_blank">', '</a>' ),
				'section'     => 'section_custom_code',
			) );
		}

		$this->wp_customize->add_control( 'custom_js_head', array(
			'type'        => 'textarea',
			'label'       => esc_html__( 'Custom JavaScript (head)', 'auto-pt' ),
			'description' => sprintf( esc_html__( 'You have to include the %s tags as well. Paste your Google Analytics tracking code here.', 'auto-pt' ), '&lt;script&gt;&lt;/script&gt;' ),
			'section'     => 'section_custom_code',
		) );

		$this->wp_customize->add_control( 'custom_js_footer', array(
			'type'        => 'textarea',
			'label'       => esc_html__( 'Custom JavaScript (footer)', 'auto-pt' ),
			'description' => sprintf( esc_html__( 'You have to include the %s tags as well.', 'auto-pt' ), '&lt;script&gt;&lt;/script&gt;' ),
			'section'     => 'section_custom_code',
		) );

		// Section: section_other.
		$this->wp_customize->add_control( 'show_acf', array(
			'type'        => 'select',
			'label'       => esc_html__( 'Show ACF admin panel?', 'auto-pt' ),
			'description' => esc_html__( 'If you want to use ACF and need the ACF admin panel set this to "Yes". Do not change if you do not know what you are doing.', 'auto-pt' ),
			'section'     => 'section_other',
			'choices'     => array(
				'no'  => esc_html__( 'No', 'auto-pt' ),
				'yes' => esc_html__( 'Yes', 'auto-pt' ),
			),
		) );
		$this->wp_customize->add_control( 'use_minified_css', array(
			'type'    => 'select',
			'label'   => esc_html__( 'Use minified theme CSS', 'auto-pt' ),
			'section' => 'section_other',
			'choices' => array(
				'no'  => esc_html__( 'No', 'auto-pt' ),
				'yes' => esc_html__( 'Yes', 'auto-pt' ),
			),
		) );
		$this->wp_customize->add_control( 'charset_setting', array(
			'type'     => 'select',
			'label'    => esc_html__( 'Character set for Google Fonts' , 'auto-pt' ),
			'section'  => 'section_other',
			'choices'  => array(
				'latin'        => 'Latin',
				'latin-ext'    => 'Latin Extended',
				'cyrillic'     => 'Cyrillic',
				'cyrillic-ext' => 'Cyrillic Extended',
				'vietnamese'   => 'Vietnamese',
				'greek'        => 'Greek',
				'greek-ext'    => 'Greek Extended',
			),
		) );
	}


	/**
	 * Partials for selective refresh
	 *
	 * @return void
	 */
	public function register_partials() {
		$this->wp_customize->selective_refresh->add_partial( 'dynamic_css', array(
			'selector' => 'head > #wp-utils-dynamic-css-style-tag',
			'settings' => array_keys( $this->dynamic_css ),
			'render_callback' => function() {
				return $this->dynamic_css_cache_manager->render_css();
			},
		) );
	}


	/**
	 * Get the dimensions of the logo image when the setting is saved.
	 * This is purely a performance improvement.
	 *
	 * Used by hook: add_action( 'customize_save_logo_img' , array( $this, 'save_logo_dimensions' ), 10, 1 );
	 *
	 * @return void
	 */
	public static function save_logo_dimensions( $setting ) {
		$logo_width_height = array();
		$img_data          = getimagesize( esc_url( $setting->post_value() ) );

		if ( is_array( $img_data ) ) {
			$logo_width_height = array_slice( $img_data, 0, 2 );
			$logo_width_height = array_combine( array( 'width', 'height' ), $logo_width_height );
		}

		set_theme_mod( 'logo_dimensions_array', $logo_width_height );
	}


	/**
	 * Create a sanitized slug, but always with a fallback to portfolio if it is empty
	 *
	 * @param  string $title
	 * @return string
	 */
	public static function sanitize_portfolio_slug( $title ) {
		return sanitize_title( $title, 'portfolio' );
	}
}
