<?php
/**
 * Helper functions
 *
 * @package auto-pt
 */

/**
 * AutoPTHelpers class with static methods
 */
class AutoPTHelpers {
	/**
	 * Get logo dimensions from the db
	 *
	 * @param  string $theme_mod theme mod where the array with width and height is saved.
	 * @return mixed             string or FALSE
	 */
	static function get_logo_dimensions( $theme_mod = 'logo_dimensions_array' ) {
		$width_height_array = get_theme_mod( $theme_mod );

		if ( is_array( $width_height_array ) && 2 === count( $width_height_array ) ) {
			return sprintf( ' width="%d" height="%d" ', absint( $width_height_array['width'] ), absint( $width_height_array['height'] ) );
		}
		else {
			return '';
		}
	}


	/**
	 * comments_number() does not use _n function, here we are to fix that
	 *
	 * @return void
	 */
	static function pretty_comments_number() {
		global $post;
		printf(
			/* translators: %s represents a number */
			_n( '%s Comment', '%s Comments', get_comments_number(), 'auto-pt' ), number_format_i18n( get_comments_number() )
		);
	}


	/**
	 * Check if WooCommerce is active
	 *
	 * @return boolean
	 */
	static function is_woocommerce_active() {
		return class_exists( 'Woocommerce' );
	}


	/**
	 * Return array of the number which represent the layout of the footer.
	 *
	 * @return array
	 */
	static function footer_widgets_layout_array() {
		$layout = get_theme_mod( 'footer_widgets_layout', '[6,9]' );
		$layout = json_decode( $layout );

		if ( is_array( $layout ) && ! empty( $layout ) ) {
			$spans = array( (int) $layout[0] );

			for ( $i = 0; $i < ( sizeof( $layout ) - 1 ); $i++ ) {
				$spans[] = $layout[ $i + 1 ] - $layout[ $i ];
			}

			$spans[] = 12 - $layout[ $i ];

			return $spans;
		}
		else if ( 1 === $layout ) { // Single column.
			return array( '12' );
		}

		// Default: disable footer.
		return array();
	}

	/**
	 * Return url with Google Fonts.
	 *
	 * @see https://github.com/grappler/wp-standard-handles/blob/master/functions.php
	 * @return string Google fonts URL for the theme.
	 */
	static function google_web_fonts_url() {
		$fonts_url = '';
		$fonts     = array();
		$subsets   = array( 'latin' );

		$fonts = apply_filters( 'autopt_pre_google_web_fonts', $fonts );

		foreach ( $fonts as $key => $value ) {
			$fonts[ $key ] = $key . ':' . implode( ',', $value );
		}

		/* translators: To add an additional character subset specific to your language, translate this to 'greek', 'cyrillic', 'devanagari' or 'vietnamese'. Do not translate into your own language. */
		$subset = esc_html_x( 'no-subset', 'Add new subset (greek, cyrillic, devanagari, vietnamese)', 'auto-pt' );
		if ( 'cyrillic' == $subset ) {
			array_push( $subsets, 'cyrillic', 'cyrillic-ext' );
		} elseif ( 'greek' == $subset ) {
			array_push( $subsets, 'greek', 'greek-ext' );
		} elseif ( 'devanagari' == $subset ) {
			array_push( $subsets, 'devanagari' );
		} elseif ( 'vietnamese' == $subset ) {
			array_push( $subsets, 'vietnamese' );
		}

		$subsets = apply_filters( 'autopt_subsets_google_web_fonts', $subsets );

		if ( $fonts ) {
			$fonts_url = add_query_arg(
				array(
					'family' => urlencode( implode( '|', $fonts ) ),
					'subset' => urlencode( implode( ',', array_unique( $subsets ) ) ),
				),
				'//fonts.googleapis.com/css'
			);
		}

		return apply_filters( 'autopt_google_web_fonts_url', $fonts_url );
	}


	/**
	 * Prepare the srcset attribute value.
	 *
	 * @param int   $img_id ID of the image.
	 * @param array $sizes array of the image sizes. Example: $sizes = array( 'jumbotron-slider-s', 'jumbotron-slider-l' );.
	 * @uses http://codex.wordpress.org/Function_Reference/wp_get_attachment_image_src
	 * @return string
	 */
	static function get_image_srcset( $img_id, $sizes ) {
		$srcset = array();

		foreach ( $sizes as $size ) {
			$img = wp_get_attachment_image_src( absint( $img_id ), $size );
			$srcset[] = sprintf( '%s %sw', $img[0], $img[1] );
		}

		return implode( ', ' , $srcset );
	}


	/**
	 * Create a style for the HTML attribute from the array of the CSS properties
	 *
	 * @param array $attrs array with CSS settings.
	 * @return string of the background style (CSS)
	 */
	static function create_background_style_attr( $attrs ) {
		$bg_style = array();

		if ( ! empty( $attrs ) ) {
			foreach ( $attrs as $key => $value ) {
				$trimmed_val = trim( $value );
				if ( ! empty( $trimmed_val ) ) {
					if ( 'background-image' === $key ) {
						$bg_style[] = $key . ': url(' . esc_url( $trimmed_val ) . ')';
					}
					else if ( 'background-color' === $key && ! array_key_exists( 'background-image', $attrs ) ) {
						// to overwrite the pattern set in customizer
						$bg_style[] = 'background: ' . $trimmed_val;
					}
					else {
						$bg_style[] = $key . ': ' . $trimmed_val;
					}
				}
			}
		}

		if ( empty( $bg_style ) ) {
			return '';
		}
		else {
			return join( '; ', $bg_style );
		}

	}


	/**
	 * Append right body classes to the body
	 *
	 * @return string
	 */
	static function add_body_class() {
		$out = array();

		if ( 'boxed' === get_theme_mod( 'layout_mode', 'wide' ) ) {
			$out[] = 'boxed';
		}

		return implode( ' ', $out );
	}


	/**
	 * Custom wp_list_comments callback (template)
	 */
	static function custom_comment( $comment, $args, $depth ) {
		$tag = ( 'div' === $args['style'] ) ? 'div' : 'li';
	?>

		<<?php echo tag_escape( $tag ); ?> id="comment-<?php comment_ID(); ?>" <?php comment_class( array( 'clearfix', empty( $args['has_children'] ) ? '' : 'parent' ) ); ?>>
			<div class="comment__content">
				<div class="comment__inner">
					<cite class="comment__author  vcard">
						<?php echo get_comment_author_link(); ?>
					</cite>
					<div class="comment__metadata">
						<time datetime="<?php comment_time( 'c' ); ?>">
							<?php comment_date( 'F j, Y' ); ?>
						</time>
					</div>
					<div class="comment__text">
						<?php if ( '0' == $comment->comment_approved ) : ?>
						<p class="comment__awaiting-moderation"><?php esc_html_e( 'Your comment is awaiting moderation.' , 'auto-pt' ); ?></p>
						<?php endif; ?>

						<?php comment_text(); ?>
					</div>
					<?php comment_reply_link( array_merge( $args, array(
						'depth' => $depth,
						'before' => '',
					) ) ); ?>
					<?php edit_comment_link( esc_html__( 'Edit', 'auto-pt' ), '' ); ?>
				</div>

		<?php
	}


	/**
	 * Is the portfolio plugin activated.
	 * Can be used only in admin
	 *
	 * @link https://codex.wordpress.org/Function_Reference/is_plugin_active
	 */
	public static function is_portfolio_plugin_active() {
		return class_exists( 'Portfolio_Post_Type' );
	}


	/**
	 * Display previous or next link on project page
	 */
	public static function get_next_prev_portfolio_link( $previous, $title ) {
		$link = get_permalink( get_adjacent_post( false, '', $previous ) );
		if ( get_permalink() != $link ) :
			?><a class="btn  portfolio__navigation-item  portfolio__navigation-item--<?php echo true === $previous ? 'left' : 'right' ?>" href="<?php echo esc_url( $link ) . '#project-navigation-anchor'; ?>">
				<?php if ( $previous ) : ?>
					<i class="fa fa-caret-left"></i> &nbsp;
				<?php endif; ?>
				<?php echo esc_html( $title ); ?>
				<?php if ( ! $previous ) : ?>
					&nbsp; <i class="fa fa-caret-right"></i>
				<?php endif; ?>
			</a><?php
		endif;
	}


	/**
	 * Helper function to get terms (categories) for custom post types
	 *
	 * @param  int    $post_id
	 * @param  string $taxonomy
	 * @return array
	 */
	public static function get_custom_categories( $post_id, $taxonomy ) {
		$out   = array();
		$terms = get_the_terms( $post_id, $taxonomy );

		if ( ! is_array( $terms ) ) {
			return array();
		}

		foreach ( $terms as $term ) {
			$out[ $term->slug ] = $term->name;
		}

		return $out;
	}


	/**
	 * Get the post excerpt. If post_excerpt data is defined use that, otherwise
	 * trim down the content to the proper size.
	 *
	 * @param string $post_excerpt
	 * @param string $post_content
	 */
	public static function get_post_excerpt( $post_excerpt, $post_content, $excerpt_length = 55 ) {
		if ( ! empty( $post_excerpt ) ) {
			return $post_excerpt;
		}

		$text = strip_shortcodes( $post_content );
		$text = wp_strip_all_tags( $text );
		$text = str_replace( ']]>', ']]&gt;', $text );
		$excerpt_more = apply_filters( 'excerpt_more', ' [&hellip;]' );
		$text = wp_trim_words( $text, $excerpt_length, $excerpt_more );

		return $text;
	}

	/**
	 * Fetch recent posts data from DB (cache it on first widget instance) and then
	 * fetch the data from cache for all other widget instances.
	 *
	 * @param string $cache_name
	 * @param int    $number_of_posts
	 */
	public static function get_cached_posts_data( $cache_name, $number_of_posts ) {
		// Get/set cache data just once for multiple widgets.
		$recent_posts_data = wp_cache_get( $cache_name );
		if ( false === $recent_posts_data ) {
			$recent_posts_original_args = array(
				'numberposts'         => $number_of_posts,
				'orderby'             => 'post_date',
				'order'               => 'DESC',
				'post_type'           => 'post',
				'post_status'         => 'publish',
				// 'suppress_filters' => false // If some WPML problems occur, uncomment this line
			);
			$recent_posts_original = wp_get_recent_posts( $recent_posts_original_args );

			// Prepare the data that we need for display.
			$recent_posts_data = array();
			foreach ( $recent_posts_original as $key => $post ) {
				$recent_posts_data[ $key ]['id']             = $post['ID'];
				$recent_posts_data[ $key ]['date']           = get_the_date( apply_filters( 'pw/widget_latest_news_grid_short_date_format', 'j M Y' ), $post['ID'] );
				$recent_posts_data[ $key ]['full_date']      = get_the_date( get_option( 'date_format' ), $post['ID'] );
				$recent_posts_data[ $key ]['full_date_time'] = get_the_date( 'c', $post['ID'] );
				$attachment_image_id                         = get_post_thumbnail_id( $post['ID'] );
				$attachment_image_data                       = wp_get_attachment_image_src( $attachment_image_id, 'large' );

				// Get featured image data if a featured image is set.
				if ( ! empty( $attachment_image_data ) ) {
					$recent_posts_data[ $key ]['image_url']    = $attachment_image_data[0];
					$recent_posts_data[ $key ]['image_width']  = $attachment_image_data[1];
					$recent_posts_data[ $key ]['image_height'] = $attachment_image_data[2];
					$recent_posts_data[ $key ]['srcset']       = self::get_image_srcset( $attachment_image_id, array( 'medium', 'large' ) );
				}

				$recent_posts_data[ $key ]['link']         = get_permalink( $post['ID'] );
				$recent_posts_data[ $key ]['title']        = $post['post_title'];
				$recent_posts_data[ $key ]['author']       = get_the_author_meta( 'display_name', $post['post_author'] );
				$recent_posts_data[ $key ]['excerpt']      = self::get_post_excerpt( $post['post_excerpt'], $post['post_content'], apply_filters( 'pw/widget_latest_news_grid_full_excerpt_length', 255 ) );
			}

			wp_cache_set( $cache_name, $recent_posts_data );
		}
		return $recent_posts_data;
	}


	/**
	 * Return correct path to the file (check child theme first)
	 *
	 * @param string $relative_file_path
	 * @return string, absolute path of the correct file.
	 */
	public static function get_correct_file_path( $relative_file_path ) {
		if ( file_exists( get_stylesheet_directory() . $relative_file_path ) ) {
			return get_stylesheet_directory() . $relative_file_path;
		}
		elseif ( file_exists( get_template_directory() . $relative_file_path ) ) {
			return get_template_directory() . $relative_file_path;
		}
		else {
			return false;
		}
	}


	/**
	 * Require the correct file with require_once (checks child theme first)
	 *
	 * @param string $relative_file_path
	 */
	public static function load_file( $relative_file_path ) {
		require_once self::get_correct_file_path( $relative_file_path );
	}


	/**
	 * Create appropriate header background style.
	 */
	public static function header_background_style() {

		// Create the header background style on all pages,
		// except on Front Page with Slider and Alternative slider page templates.
		if ( ! is_page_template( array( 'template-front-page-slider.php', 'template-front-page-slider-alt.php' ) ) ) {
			// Regular page id.
			$auto_bg_id = get_the_ID();

			// If blog or single post use the ID of the blog.
			if ( is_home() || is_singular( 'post' ) ) {
				$auto_bg_id = absint( get_option( 'page_for_posts' ) );
			}

			// If woocommerce page, use the shop page id.
			if ( self::is_woocommerce_active() && is_woocommerce() ) {
				$auto_bg_id = absint( get_option( 'woocommerce_shop_page_id', 0 ) );
			}

			// If Portfolio widget is activate use the parent page for single portfolio pages.
			if ( self::is_portfolio_plugin_active() && is_singular( 'portfolio' ) ) {
				$auto_bg_id = absint( get_theme_mod( 'portfolio_parent_page', 0 ) );
			}

			$auto_style_array = array();

			if ( $background_image = get_field( 'background_image', $auto_bg_id ) ) {
				$auto_style_array = array(
					'background-image'      => $background_image,
					'background-position'   => get_field( 'background_image_horizontal_position', $auto_bg_id ) . ' ' . get_field( 'background_image_vertical_position', $auto_bg_id ),
					'background-repeat'     => get_field( 'background_image_repeat', $auto_bg_id ),
					'background-attachment' => get_field( 'background_image_attachment', $auto_bg_id ),
				);
			}

			$auto_style_array['background-color'] = get_field( 'background_color', $auto_bg_id );

			// Create the style tag to use in the .site-header element.
			return self::create_background_style_attr( $auto_style_array );
		}

		return '';
	}
}
